@@ -784,7 +784,12 @@ Status OlapScanNode::build_key_ranges_and_filters() {
 
         RETURN_IF_ERROR(std::visit(
                 [&](auto&& range) {
-                    RETURN_IF_ERROR(_scan_keys.extend_scan_key(range, _max_scan_key_num,
+                    // make a copy or range and pass to extend_scan_key, keep the range unchanged
+                    // because extend_scan_key method may change the first parameter.
+                    // but the original range may be converted to olap filters, if it's not an exact_range.
+                    // related pr https://github.com/apache/doris/pull/13530
+                    auto temp_range = range;
+                    RETURN_IF_ERROR(_scan_keys.extend_scan_key(temp_range, _max_scan_key_num,
                                                                &exact_range, &eos));
                     if (exact_range) {
                         _column_value_ranges.erase(iter->first);
