@@ -14,28 +14,25 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/be/src/exec/partitioned-hash-table.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_EXEC_NEW_PARTITIONED_HASH_TABLE_H
-#define DORIS_BE_SRC_EXEC_NEW_PARTITIONED_HASH_TABLE_H
+#pragma once
 
 #include <memory>
 #include <vector>
 
 #include "common/compiler_util.h"
-#include "common/logging.h"
 #include "runtime/buffered_tuple_stream3.h"
-#include "runtime/buffered_tuple_stream3.inline.h"
-#include "runtime/bufferpool/buffer_pool.h"
 #include "runtime/bufferpool/suballocator.h"
 #include "runtime/tuple_row.h"
 #include "util/bitmap.h"
-#include "util/hash_util.hpp"
 
 namespace doris {
 
 class Expr;
 class ExprContext;
-class MemTracker;
 class PartitionedHashTable;
 class RowDescriptor;
 class RuntimeState;
@@ -112,8 +109,7 @@ public:
                          const std::vector<Expr*>& probe_exprs, bool stores_nulls,
                          const std::vector<bool>& finds_nulls, int32_t initial_seed, int max_levels,
                          int num_build_tuples, MemPool* mem_pool, MemPool* expr_results_pool,
-                         const std::shared_ptr<MemTracker>& tracker, const RowDescriptor& row_desc,
-                         const RowDescriptor& row_desc_probe,
+                         const RowDescriptor& row_desc, const RowDescriptor& row_desc_probe,
                          std::unique_ptr<PartitionedHashTableCtx>* ht_ctx);
 
     /// Initialize the build and probe expression evaluators.
@@ -206,13 +202,10 @@ public:
 
         /// Allocates memory and initializes various data structures. Return error status
         /// if memory allocation leads to the memory limits of the exec node to be exceeded.
-        /// 'tracker' is the memory tracker of the exec node which owns this PartitionedHashTableCtx.
-        Status Init(RuntimeState* state, const std::shared_ptr<MemTracker>& tracker,
-                    const std::vector<Expr*>& build_exprs);
+        Status Init(RuntimeState* state, const std::vector<Expr*>& build_exprs);
 
-        /// Frees up various resources and updates memory tracker with proper accounting.
-        /// 'tracker' should be the same memory tracker which was passed in for Init().
-        void Close(const std::shared_ptr<MemTracker>& tracker);
+        /// Frees up various resources.
+        void Close();
 
         /// Resets the cache states (iterators, end pointers etc) before writing.
         void Reset() noexcept;
@@ -376,8 +369,7 @@ private:
     PartitionedHashTableCtx(const std::vector<Expr*>& build_exprs,
                             const std::vector<Expr*>& probe_exprs, bool stores_nulls,
                             const std::vector<bool>& finds_nulls, int32_t initial_seed,
-                            int max_levels, MemPool* mem_pool, MemPool* expr_results_pool,
-                            const std::shared_ptr<MemTracker>& tracker);
+                            int max_levels, MemPool* mem_pool, MemPool* expr_results_pool);
 
     /// Allocate various buffers for storing expression evaluation results, hash values,
     /// null bits etc. Also allocate evaluators for the build and probe expressions and
@@ -443,8 +435,6 @@ private:
     bool stores_nulls() const { return stores_nulls_; }
     bool finds_some_nulls() const { return finds_some_nulls_; }
 
-    std::shared_ptr<MemTracker> tracker_;
-
     const std::vector<Expr*>& build_exprs_;
     std::vector<ExprContext*> build_expr_evals_;
 
@@ -964,5 +954,3 @@ private:
 };
 
 } // namespace doris
-
-#endif
