@@ -46,7 +46,9 @@ template BooleanVal IsNullPredicate::is_null(FunctionContext*, const LargeIntVal
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const FloatVal&);
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const DoubleVal&);
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const StringVal&);
+template BooleanVal IsNullPredicate::is_null(FunctionContext*, const DateV2Val&);
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const DateTimeVal&);
+template BooleanVal IsNullPredicate::is_null(FunctionContext*, const DateTimeV2Val&);
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const DecimalV2Val&);
 template BooleanVal IsNullPredicate::is_null(FunctionContext*, const CollectionVal&);
 
@@ -60,7 +62,9 @@ template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const LargeIn
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const FloatVal&);
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const DoubleVal&);
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const StringVal&);
+template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const DateV2Val&);
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const DateTimeVal&);
+template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const DateTimeV2Val&);
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const DecimalV2Val&);
 template BooleanVal IsNullPredicate::is_not_null(FunctionContext*, const CollectionVal&);
 
