@@ -17,12 +17,12 @@
 // (3) (Optional) Replace old Split*() API calls at call sites with calls to new
 //     Split() API.
 //
-#ifndef STRINGS_SPLIT_H_
-#define STRINGS_SPLIT_H_
+#pragma once
 
 #include <stddef.h>
 
 #include <algorithm>
+
 using std::copy;
 using std::max;
 using std::min;
@@ -1143,5 +1143,3 @@ bool SplitStringAndParseToList(StringPiece source, StringPiece delim,
 
 // END DOXYGEN SplitFunctions grouping
 /* @} */
-
-#endif // STRINGS_SPLIT_H_
