@@ -170,9 +170,8 @@ void StorageEngine::_fd_cache_clean_callback() {
 #ifdef GOOGLE_PROFILER
     ProfilerRegisterThread();
 #endif
-    int32_t interval = 600;
+    int32_t interval = config::cache_clean_interval;
     while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval))) {
-        interval = config::cache_clean_interval;
         if (interval <= 0) {
             LOG(WARNING) << "config of file descriptor clean interval is illegal: [" << interval
                          << "], force set to 3600 ";
