@@ -22,7 +22,6 @@
 #include "common/logging.h"
 #include "env/env.h"
 #include "gutil/strings/substitute.h"
-#include "olap/fs/block_manager.h"
 #include "olap/rowset/segment_v2/bitmap_index_writer.h"
 #include "olap/rowset/segment_v2/bloom_filter.h"
 #include "olap/rowset/segment_v2/bloom_filter_index_writer.h"
@@ -77,12 +76,12 @@ private:
 };
 
 Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn* column,
-                            fs::WritableBlock* _wblock, std::unique_ptr<ColumnWriter>* writer) {
+                            io::FileWriter* file_writer, std::unique_ptr<ColumnWriter>* writer) {
     std::unique_ptr<Field> field(FieldFactory::create(*column));
     DCHECK(field.get() != nullptr);
     if (is_scalar_type(column->type())) {
         std::unique_ptr<ColumnWriter> writer_local = std::unique_ptr<ColumnWriter>(
-                new ScalarColumnWriter(opts, std::move(field), _wblock));
+                new ScalarColumnWriter(opts, std::move(field), file_writer));
         *writer = std::move(writer_local);
         return Status::OK();
     } else {
@@ -107,10 +106,10 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
             }
             std::unique_ptr<ColumnWriter> item_writer;
             RETURN_IF_ERROR(
-                    ColumnWriter::create(item_options, &item_column, _wblock, &item_writer));
+                    ColumnWriter::create(item_options, &item_column, file_writer, &item_writer));
 
             // create length writer
-            FieldType length_type = FieldType::OLAP_FIELD_TYPE_UNSIGNED_INT;
+            FieldType length_type = FieldType::OLAP_FIELD_TYPE_UNSIGNED_BIGINT;
 
             ColumnWriterOptions length_options;
             length_options.meta = opts.meta->add_children_columns();
@@ -118,7 +117,8 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
             length_options.meta->set_unique_id(2);
             length_options.meta->set_type(length_type);
             length_options.meta->set_is_nullable(false);
-            length_options.meta->set_length(get_scalar_type_info<OLAP_FIELD_TYPE_UNSIGNED_INT>()->size());
+            length_options.meta->set_length(
+                    get_scalar_type_info<OLAP_FIELD_TYPE_UNSIGNED_BIGINT>()->size());
             length_options.meta->set_encoding(DEFAULT_ENCODING);
             length_options.meta->set_compression(opts.meta->compression());
 
@@ -133,7 +133,7 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
             length_column.set_index_length(-1); // no short key index
             std::unique_ptr<Field> bigint_field(FieldFactory::create(length_column));
             auto* length_writer =
-                    new ScalarColumnWriter(length_options, std::move(bigint_field), _wblock);
+                    new ScalarColumnWriter(length_options, std::move(bigint_field), file_writer);
 
             // if nullable, create null writer
             ScalarColumnWriter* null_writer = nullptr;
@@ -145,7 +145,8 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
                 null_options.meta->set_unique_id(3);
                 null_options.meta->set_type(null_type);
                 null_options.meta->set_is_nullable(false);
-                null_options.meta->set_length(get_scalar_type_info<OLAP_FIELD_TYPE_TINYINT>()->size());
+                null_options.meta->set_length(
+                        get_scalar_type_info<OLAP_FIELD_TYPE_TINYINT>()->size());
                 null_options.meta->set_encoding(DEFAULT_ENCODING);
                 null_options.meta->set_compression(opts.meta->compression());
 
@@ -159,7 +160,8 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
                 null_column.set_name("nullable");
                 null_column.set_index_length(-1); // no short key index
                 std::unique_ptr<Field> null_field(FieldFactory::create(null_column));
-                null_writer = new ScalarColumnWriter(null_options, std::move(null_field), _wblock);
+                null_writer =
+                        new ScalarColumnWriter(null_options, std::move(null_field), file_writer);
             }
 
             std::unique_ptr<ColumnWriter> writer_local = std::unique_ptr<ColumnWriter>(
@@ -169,7 +171,7 @@ Status ColumnWriter::create(const ColumnWriterOptions& opts, const TabletColumn*
             return Status::OK();
         }
         default:
-            return Status::NotSupported("unsupported type for ColumnWriter: " +
+            return Status::NotSupported("unsupported type for ColumnWriter: {}",
                                         std::to_string(field->type()));
         }
     }
@@ -191,13 +193,54 @@ Status ColumnWriter::append_nullable(const uint8_t* is_null_bits, const void* da
     return Status::OK();
 }
 
+Status ColumnWriter::append_nullable(const uint8_t* null_map, const uint8_t** ptr,
+                                     size_t num_rows) {
+    size_t offset = 0;
+    auto next_run_step = [&]() {
+        size_t step = 1;
+        for (auto i = offset + 1; i < num_rows; ++i) {
+            if (null_map[offset] == null_map[i])
+                step++;
+            else
+                break;
+        }
+        return step;
+    };
+
+    do {
+        auto step = next_run_step();
+        if (null_map[offset]) {
+            RETURN_IF_ERROR(append_nulls(step));
+            *ptr += get_field()->size() * step;
+        } else {
+            // TODO:
+            //  1. `*ptr += get_field()->size() * step;` should do in this function, not append_data;
+            //  2. support array vectorized load and ptr offset add
+            RETURN_IF_ERROR(append_data(ptr, step));
+        }
+        offset += step;
+    } while (offset < num_rows);
+
+    return Status::OK();
+}
+
+Status ColumnWriter::append(const uint8_t* nullmap, const void* data, size_t num_rows) {
+    assert(data && num_rows > 0);
+    const auto* ptr = (const uint8_t*)data;
+    if (nullmap) {
+        return append_nullable(nullmap, &ptr, num_rows);
+    } else {
+        return append_data(&ptr, num_rows);
+    }
+}
+
 ///////////////////////////////////////////////////////////////////////////////////
 
 ScalarColumnWriter::ScalarColumnWriter(const ColumnWriterOptions& opts,
-                                       std::unique_ptr<Field> field, fs::WritableBlock* wblock)
+                                       std::unique_ptr<Field> field, io::FileWriter* file_writer)
         : ColumnWriter(std::move(field), opts.meta->is_nullable()),
           _opts(opts),
-          _wblock(wblock),
+          _file_writer(file_writer),
           _data_size(0) {
     // these opts.meta fields should be set by client
     DCHECK(opts.meta->has_column_id());
@@ -207,7 +250,7 @@ ScalarColumnWriter::ScalarColumnWriter(const ColumnWriterOptions& opts,
     DCHECK(opts.meta->has_encoding());
     DCHECK(opts.meta->has_compression());
     DCHECK(opts.meta->has_is_nullable());
-    DCHECK(wblock != nullptr);
+    DCHECK(file_writer != nullptr);
 }
 
 ScalarColumnWriter::~ScalarColumnWriter() {
@@ -221,7 +264,7 @@ ScalarColumnWriter::~ScalarColumnWriter() {
 }
 
 Status ScalarColumnWriter::init() {
-    RETURN_IF_ERROR(get_block_compression_codec(_opts.meta->compression(), _compress_codec));
+    RETURN_IF_ERROR(get_block_compression_codec(_opts.meta->compression(), &_compress_codec));
 
     PageBuilder* page_builder = nullptr;
 
@@ -233,9 +276,8 @@ Status ScalarColumnWriter::init() {
     opts.data_page_size = _opts.data_page_size;
     RETURN_IF_ERROR(_encoding_info->create_page_builder(opts, &page_builder));
     if (page_builder == nullptr) {
-        return Status::NotSupported(
-                strings::Substitute("Failed to create page builder for type $0 and encoding $1",
-                                    get_field()->type(), _opts.meta->encoding()));
+        return Status::NotSupported("Failed to create page builder for type {} and encoding {}",
+                                    get_field()->type(), _opts.meta->encoding());
     }
     // should store more concrete encoding type instead of DEFAULT_ENCODING
     // because the default encoding of a data type can be changed in the future
@@ -248,7 +290,7 @@ Status ScalarColumnWriter::init() {
         _null_bitmap_builder.reset(new NullBitmapBuilder());
     }
     if (_opts.need_zone_map) {
-        _zone_map_index_builder.reset(new ZoneMapIndexWriter(get_field()));
+        RETURN_IF_ERROR(ZoneMapIndexWriter::create(get_field(), _zone_map_index_builder));
     }
     if (_opts.need_bitmap_index) {
         RETURN_IF_ERROR(
@@ -294,20 +336,20 @@ Status ScalarColumnWriter::append_data(const uint8_t** ptr, size_t num_rows) {
     return Status::OK();
 }
 
-Status ScalarColumnWriter::append_data_in_current_page(const uint8_t** ptr, size_t* num_written) {
-    RETURN_IF_ERROR(_page_builder->add(*ptr, num_written));
+Status ScalarColumnWriter::append_data_in_current_page(const uint8_t* data, size_t* num_written) {
+    RETURN_IF_ERROR(_page_builder->add(data, num_written));
     if (_opts.need_zone_map) {
-        _zone_map_index_builder->add_values(*ptr, *num_written);
+        _zone_map_index_builder->add_values(data, *num_written);
     }
     if (_opts.need_bitmap_index) {
-        _bitmap_index_builder->add_values(*ptr, *num_written);
+        _bitmap_index_builder->add_values(data, *num_written);
     }
     if (_opts.need_bloom_filter) {
-        _bloom_filter_index_builder->add_values(*ptr, *num_written);
+        _bloom_filter_index_builder->add_values(data, *num_written);
     }
 
     _next_rowid += *num_written;
-    *ptr += get_field()->size() * (*num_written);
+
     // we must write null bits after write data, because we don't
     // know how many rows can be written into current page
     if (is_nullable()) {
@@ -316,22 +358,9 @@ Status ScalarColumnWriter::append_data_in_current_page(const uint8_t** ptr, size
     return Status::OK();
 }
 
-Status ScalarColumnWriter::append_data_in_current_page(const uint8_t* ptr, size_t* num_written) {
-    RETURN_IF_ERROR(_page_builder->add(ptr, num_written));
-    if (_opts.need_zone_map) {
-        _zone_map_index_builder->add_values(ptr, *num_written);
-    }
-    if (_opts.need_bitmap_index) {
-        _bitmap_index_builder->add_values(ptr, *num_written);
-    }
-    if (_opts.need_bloom_filter) {
-        _bloom_filter_index_builder->add_values(ptr, *num_written);
-    }
-
-    _next_rowid += *num_written;
-    if (is_nullable()) {
-        _null_bitmap_builder->add_run(false, *num_written);
-    }
+Status ScalarColumnWriter::append_data_in_current_page(const uint8_t** data, size_t* num_written) {
+    RETURN_IF_ERROR(append_data_in_current_page(*data, num_written));
+    *data += get_field()->size() * (*num_written);
     return Status::OK();
 }
 
@@ -377,35 +406,35 @@ Status ScalarColumnWriter::write_data() {
         footer.mutable_dict_page_footer()->set_encoding(PLAIN_ENCODING);
 
         PagePointer dict_pp;
-        RETURN_IF_ERROR(PageIO::compress_and_write_page(_compress_codec.get(),
-                                                        _opts.compression_min_space_saving, _wblock,
-                                                        {dict_body.slice()}, footer, &dict_pp));
+        RETURN_IF_ERROR(PageIO::compress_and_write_page(
+                _compress_codec, _opts.compression_min_space_saving, _file_writer,
+                {dict_body.slice()}, footer, &dict_pp));
         dict_pp.to_proto(_opts.meta->mutable_dict_page());
     }
     return Status::OK();
 }
 
 Status ScalarColumnWriter::write_ordinal_index() {
-    return _ordinal_index_builder->finish(_wblock, _opts.meta->add_indexes());
+    return _ordinal_index_builder->finish(_file_writer, _opts.meta->add_indexes());
 }
 
 Status ScalarColumnWriter::write_zone_map() {
     if (_opts.need_zone_map) {
-        return _zone_map_index_builder->finish(_wblock, _opts.meta->add_indexes());
+        return _zone_map_index_builder->finish(_file_writer, _opts.meta->add_indexes());
     }
     return Status::OK();
 }
 
 Status ScalarColumnWriter::write_bitmap_index() {
     if (_opts.need_bitmap_index) {
-        return _bitmap_index_builder->finish(_wblock, _opts.meta->add_indexes());
+        return _bitmap_index_builder->finish(_file_writer, _opts.meta->add_indexes());
     }
     return Status::OK();
 }
 
 Status ScalarColumnWriter::write_bloom_filter_index() {
     if (_opts.need_bloom_filter) {
-        return _bloom_filter_index_builder->finish(_wblock, _opts.meta->add_indexes());
+        return _bloom_filter_index_builder->finish(_file_writer, _opts.meta->add_indexes());
     }
     return Status::OK();
 }
@@ -417,7 +446,7 @@ Status ScalarColumnWriter::_write_data_page(Page* page) {
     for (auto& data : page->data) {
         compressed_body.push_back(data.slice());
     }
-    RETURN_IF_ERROR(PageIO::write_page(_wblock, compressed_body, page->footer, &pp));
+    RETURN_IF_ERROR(PageIO::write_page(_file_writer, compressed_body, page->footer, &pp));
     _ordinal_index_builder->append_entry(page->footer.data_page_footer().first_ordinal(), pp);
     return Status::OK();
 }
@@ -465,8 +494,8 @@ Status ScalarColumnWriter::finish_current_page() {
     }
     // trying to compress page body
     OwnedSlice compressed_body;
-    RETURN_IF_ERROR(PageIO::compress_page_body(
-            _compress_codec.get(), _opts.compression_min_space_saving, body, &compressed_body));
+    RETURN_IF_ERROR(PageIO::compress_page_body(_compress_codec, _opts.compression_min_space_saving,
+                                               body, &compressed_body));
     if (compressed_body.slice().empty()) {
         // page body is uncompressed
         page->data.emplace_back(std::move(encoded_values));
@@ -484,30 +513,31 @@ Status ScalarColumnWriter::finish_current_page() {
 ////////////////////////////////////////////////////////////////////////////////
 
 ArrayColumnWriter::ArrayColumnWriter(const ColumnWriterOptions& opts, std::unique_ptr<Field> field,
-                                     ScalarColumnWriter* length_writer,
+                                     ScalarColumnWriter* offset_writer,
                                      ScalarColumnWriter* null_writer,
                                      std::unique_ptr<ColumnWriter> item_writer)
         : ColumnWriter(std::move(field), opts.meta->is_nullable()),
           _item_writer(std::move(item_writer)),
           _opts(opts) {
-    _length_writer.reset(length_writer);
+    _offset_writer.reset(offset_writer);
     if (is_nullable()) {
         _null_writer.reset(null_writer);
     }
 }
 
 Status ArrayColumnWriter::init() {
-    RETURN_IF_ERROR(_length_writer->init());
+    RETURN_IF_ERROR(_offset_writer->init());
     if (is_nullable()) {
         RETURN_IF_ERROR(_null_writer->init());
     }
     RETURN_IF_ERROR(_item_writer->init());
-    _length_writer->register_flush_page_callback(this);
+    _offset_writer->register_flush_page_callback(this);
+
     return Status::OK();
 }
 
 Status ArrayColumnWriter::put_extra_info_in_page(DataPageFooterPB* footer) {
-    footer->set_first_array_item_ordinal(_current_length_page_first_ordinal);
+    footer->set_next_array_item_ordinal(_item_writer->get_next_rowid());
     return Status::OK();
 }
 
@@ -518,14 +548,12 @@ Status ArrayColumnWriter::append_data(const uint8_t** ptr, size_t num_rows) {
     while (remaining > 0) {
         // TODO llj: bulk write
         size_t num_written = 1;
-        auto size_ptr = col_cursor->length();
-        RETURN_IF_ERROR(_length_writer->append_data_in_current_page(
-                reinterpret_cast<uint8_t*>(&size_ptr), &num_written));
+        ordinal_t next_item_ordinal = _item_writer->get_next_rowid();
+        RETURN_IF_ERROR(_offset_writer->append_data_in_current_page(
+                reinterpret_cast<uint8_t*>(&next_item_ordinal), &num_written));
         if (num_written <
             1) { // page is full, write first item offset and update current length page's start ordinal
-            RETURN_IF_ERROR(_length_writer->finish_current_page());
-            _current_length_page_first_ordinal += _lengh_sum_in_cur_page;
-            _lengh_sum_in_cur_page = 0;
+            RETURN_IF_ERROR(_offset_writer->finish_current_page());
         } else {
             // write child item.
             if (_item_writer->is_nullable()) {
@@ -539,11 +567,12 @@ Status ArrayColumnWriter::append_data(const uint8_t** ptr, size_t num_rows) {
                 RETURN_IF_ERROR(_item_writer->append_data(reinterpret_cast<const uint8_t**>(&data),
                                                           col_cursor->length()));
             }
-            _lengh_sum_in_cur_page += col_cursor->length();
         }
         remaining -= num_written;
         col_cursor += num_written;
+        *ptr += num_written * sizeof(CollectionValue);
     }
+
     if (is_nullable()) {
         return write_null_column(num_rows, false);
     }
@@ -551,13 +580,13 @@ Status ArrayColumnWriter::append_data(const uint8_t** ptr, size_t num_rows) {
 }
 
 uint64_t ArrayColumnWriter::estimate_buffer_size() {
-    return _length_writer->estimate_buffer_size() +
+    return _offset_writer->estimate_buffer_size() +
            (is_nullable() ? _null_writer->estimate_buffer_size() : 0) +
            _item_writer->estimate_buffer_size();
 }
 
 Status ArrayColumnWriter::finish() {
-    RETURN_IF_ERROR(_length_writer->finish());
+    RETURN_IF_ERROR(_offset_writer->finish());
     if (is_nullable()) {
         RETURN_IF_ERROR(_null_writer->finish());
     }
@@ -566,7 +595,7 @@ Status ArrayColumnWriter::finish() {
 }
 
 Status ArrayColumnWriter::write_data() {
-    RETURN_IF_ERROR(_length_writer->write_data());
+    RETURN_IF_ERROR(_offset_writer->write_data());
     if (is_nullable()) {
         RETURN_IF_ERROR(_null_writer->write_data());
     }
@@ -575,21 +604,23 @@ Status ArrayColumnWriter::write_data() {
 }
 
 Status ArrayColumnWriter::write_ordinal_index() {
-    RETURN_IF_ERROR(_length_writer->write_ordinal_index());
+    RETURN_IF_ERROR(_offset_writer->write_ordinal_index());
     if (is_nullable()) {
         RETURN_IF_ERROR(_null_writer->write_ordinal_index());
     }
-    RETURN_IF_ERROR(_item_writer->write_ordinal_index());
+    if (!has_empty_items()) {
+        RETURN_IF_ERROR(_item_writer->write_ordinal_index());
+    }
     return Status::OK();
 }
 
 Status ArrayColumnWriter::append_nulls(size_t num_rows) {
     size_t num_lengths = num_rows;
-    const ordinal_t zero = 0;
+    const ordinal_t offset = _item_writer->get_next_rowid();
     while (num_lengths > 0) {
         // TODO llj bulk write
-        const auto* zero_ptr = reinterpret_cast<const uint8_t*>(&zero);
-        RETURN_IF_ERROR(_length_writer->append_data(&zero_ptr, 1));
+        const auto* offset_ptr = reinterpret_cast<const uint8_t*>(&offset);
+        RETURN_IF_ERROR(_offset_writer->append_data(&offset_ptr, 1));
         --num_lengths;
     }
     return write_null_column(num_rows, true);
