@@ -57,20 +57,13 @@ public:
     void add_error_tablet_id(int64_t tablet_id);
     void add_succ_tablet_id(int64_t tablet_id);
 
-    void notify();
-    void wait();
-
     int64_t finish_task();
 
 private:
-    std::atomic<int64_t> _total_task_num;
     const TPublishVersionRequest& _publish_version_req;
     std::mutex _tablet_ids_mutex;
     vector<TTabletId>* _error_tablet_ids;
     vector<TTabletId>* _succ_tablet_ids;
-
-    std::mutex _tablet_finish_mutex;
-    std::condition_variable _tablet_finish_cond;
 };
 
 } // namespace doris
