@@ -14,9 +14,11 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.10.0/be/src/runtime/buffered-tuple-stream.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_RUNTIME_BUFFERED_TUPLE_STREAM2_H
-#define DORIS_BE_SRC_RUNTIME_BUFFERED_TUPLE_STREAM2_H
+#pragma once
 
 #include <vector>
 
@@ -408,5 +410,3 @@ private:
 };
 
 } // end namespace doris
-
-#endif // DORIS_BE_SRC_RUNTIME_BUFFERED_TUPLE_STREAM2_H
