@@ -35,14 +35,15 @@
 #include <ctime>
 #include <memory>
 
-#include "common/logging.h"
 #include "exprs/slot_ref.h"
 #include "gutil/strings/substitute.h"
 #include "runtime/descriptor_helper.h"
 #include "runtime/descriptors.h"
+#include "runtime/large_int_value.h"
 #include "runtime/row_batch.h"
 #include "util/arrow/utils.h"
 #include "util/types.h"
+#include "vec/runtime/vdatetime_value.h"
 
 namespace doris {
 
@@ -76,8 +77,11 @@ Status convert_to_arrow_type(const TypeDescriptor& type, std::shared_ptr<arrow::
     case TYPE_HLL:
     case TYPE_LARGEINT:
     case TYPE_DATE:
+    case TYPE_DATEV2:
     case TYPE_DATETIME:
+    case TYPE_DATETIMEV2:
     case TYPE_STRING:
+    case TYPE_OBJECT:
         *result = arrow::utf8();
         break;
     case TYPE_DECIMALV2:
@@ -87,8 +91,7 @@ Status convert_to_arrow_type(const TypeDescriptor& type, std::shared_ptr<arrow::
         *result = arrow::boolean();
         break;
     default:
-        return Status::InvalidArgument(
-                strings::Substitute("Unknown primitive type($0)", type.type));
+        return Status::InvalidArgument("Unknown primitive type({})", type.type);
     }
     return Status::OK();
 }
@@ -138,7 +141,7 @@ Status convert_to_doris_type(const arrow::DataType& type, TSlotDescriptorBuilder
         builder->type(TYPE_BOOLEAN);
         break;
     default:
-        return Status::InvalidArgument(strings::Substitute("Unknown arrow type id($0)", type.id()));
+        return Status::InvalidArgument("Unknown arrow type id({})", type.id());
     }
     return Status::OK();
 }
@@ -219,6 +222,7 @@ public:
             case TYPE_VARCHAR:
             case TYPE_CHAR:
             case TYPE_HLL:
+            case TYPE_OBJECT:
             case TYPE_STRING: {
                 const StringValue* string_val = (const StringValue*)(cell_ptr);
                 if (string_val->len == 0) {
@@ -230,6 +234,22 @@ public:
                 }
                 break;
             }
+            case TYPE_DATEV2: {
+                const vectorized::DateV2Value<vectorized::DateV2ValueType>* date_val =
+                        (const vectorized::DateV2Value<vectorized::DateV2ValueType>*)(cell_ptr);
+                char buf[64];
+                int len = date_val->to_buffer(buf);
+                ARROW_RETURN_NOT_OK(builder.Append(buf, len));
+                break;
+            }
+            case TYPE_DATETIMEV2: {
+                const vectorized::DateV2Value<vectorized::DateTimeV2ValueType>* datetime_val =
+                        (const vectorized::DateV2Value<vectorized::DateTimeV2ValueType>*)(cell_ptr);
+                char buf[64];
+                int len = datetime_val->to_buffer(buf);
+                ARROW_RETURN_NOT_OK(builder.Append(buf, len));
+                break;
+            }
             case TYPE_DATE:
             case TYPE_DATETIME: {
                 char buf[64];
@@ -366,9 +386,8 @@ class ToRowBatchConverter : public arrow::ArrayVisitor {
 public:
     using arrow::ArrayVisitor::Visit;
 
-    ToRowBatchConverter(const arrow::RecordBatch& batch, const RowDescriptor& row_desc,
-                        const std::shared_ptr<MemTracker>& tracker)
-            : _batch(batch), _row_desc(row_desc), _tracker(tracker) {}
+    ToRowBatchConverter(const arrow::RecordBatch& batch, const RowDescriptor& row_desc)
+            : _batch(batch), _row_desc(row_desc) {}
 
 #define PRIMITIVE_VISIT(TYPE) \
     arrow::Status Visit(const arrow::TYPE& array) override { return _visit(array); }
@@ -408,7 +427,6 @@ private:
 private:
     const arrow::RecordBatch& _batch;
     const RowDescriptor& _row_desc;
-    std::shared_ptr<MemTracker> _tracker;
 
     std::unique_ptr<SlotRef> _cur_slot_ref;
     std::shared_ptr<RowBatch> _output;
@@ -428,7 +446,7 @@ Status ToRowBatchConverter::convert(std::shared_ptr<RowBatch>* result) {
     // TODO(zc): check if field type match
 
     size_t num_rows = _batch.num_rows();
-    _output.reset(new RowBatch(_row_desc, num_rows, _tracker.get()));
+    _output.reset(new RowBatch(_row_desc, num_rows));
     _output->commit_rows(num_rows);
     auto pool = _output->tuple_data_pool();
     for (size_t row_id = 0; row_id < num_rows; ++row_id) {
@@ -454,9 +472,8 @@ Status ToRowBatchConverter::convert(std::shared_ptr<RowBatch>* result) {
 }
 
 Status convert_to_row_batch(const arrow::RecordBatch& batch, const RowDescriptor& row_desc,
-                            const std::shared_ptr<MemTracker>& tracker,
                             std::shared_ptr<RowBatch>* result) {
-    ToRowBatchConverter converter(batch, row_desc, tracker);
+    ToRowBatchConverter converter(batch, row_desc);
     return converter.convert(result);
 }
 
@@ -465,51 +482,40 @@ Status serialize_record_batch(const arrow::RecordBatch& record_batch, std::strin
     int64_t capacity;
     arrow::Status a_st = arrow::ipc::GetRecordBatchSize(record_batch, &capacity);
     if (!a_st.ok()) {
-        std::stringstream msg;
-        msg << "GetRecordBatchSize failure, reason: " << a_st.ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("GetRecordBatchSize failure, reason: {}", a_st.ToString());
     }
     auto sink_res = arrow::io::BufferOutputStream::Create(capacity, arrow::default_memory_pool());
     if (!sink_res.ok()) {
-        std::stringstream msg;
-        msg << "create BufferOutputStream failure, reason: " << sink_res.status().ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("create BufferOutputStream failure, reason: {}",
+                                     sink_res.status().ToString());
     }
     std::shared_ptr<arrow::io::BufferOutputStream> sink = sink_res.ValueOrDie();
     // create RecordBatch Writer
     auto res = arrow::ipc::MakeStreamWriter(sink.get(), record_batch.schema());
     if (!res.ok()) {
-        std::stringstream msg;
-        msg << "open RecordBatchStreamWriter failure, reason: " << res.status().ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("open RecordBatchStreamWriter failure, reason: {}",
+                                     res.status().ToString());
     }
     // write RecordBatch to memory buffer outputstream
     std::shared_ptr<arrow::ipc::RecordBatchWriter> record_batch_writer = res.ValueOrDie();
     a_st = record_batch_writer->WriteRecordBatch(record_batch);
     if (!a_st.ok()) {
-        std::stringstream msg;
-        msg << "write record batch failure, reason: " << a_st.ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("write record batch failure, reason: {}", a_st.ToString());
     }
     a_st = record_batch_writer->Close();
     if (!a_st.ok()) {
-        std::stringstream msg;
-        msg << "Close failed, reason: " << a_st.ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("Close failed, reason: {}", a_st.ToString());
     }
     auto finish_res = sink->Finish();
     if (!finish_res.ok()) {
-        std::stringstream msg;
-        msg << "allocate result buffer failure, reason: " << finish_res.status().ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("allocate result buffer failure, reason: {}",
+                                     finish_res.status().ToString());
     }
     *result = finish_res.ValueOrDie()->ToString();
     // close the sink
     a_st = sink->Close();
     if (!a_st.ok()) {
-        std::stringstream msg;
-        msg << "Close failed, reason: " << a_st.ToString();
-        return Status::InternalError(msg.str());
+        return Status::InternalError("Close failed, reason: {}", a_st.ToString());
     }
     return Status::OK();
 }
