@@ -14,9 +14,11 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/be/src/util/dummy-runtime-profile.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_UTIL_DEBUG_RUNTIME_PROFILE_H
-#define DORIS_BE_SRC_UTIL_DEBUG_RUNTIME_PROFILE_H
+#pragma once
 
 #include "common/object_pool.h"
 #include "util/runtime_profile.h"
@@ -33,4 +35,3 @@ private:
     RuntimeProfile* const _profile;
 };
 } // namespace doris
-#endif
