@@ -68,7 +68,9 @@ struct ParquetInt96 {
     uint64_t lo; // time of nanoseconds in a day
     uint32_t hi; // days from julian epoch
 
-    inline uint64_t to_timestamp_micros() const;
+    inline uint64_t to_timestamp_micros() const {
+        return (hi - JULIAN_EPOCH_OFFSET_DAYS) * MICROS_IN_DAY + lo / NANOS_PER_MICROSECOND;
+    }
 
     static const uint32_t JULIAN_EPOCH_OFFSET_DAYS;
     static const uint64_t MICROS_IN_DAY;
@@ -361,7 +363,6 @@ Status FixLengthDecoder::_decode_datetime64(MutableColumnPtr& doris_column,
     size_t data_index = column_data.size();
     column_data.resize(data_index + select_vector.num_values() - select_vector.num_filtered());
     size_t dict_index = 0;
-    int64_t scale_to_micro = _decode_params->scale_to_nano_factor / 1000;
     ColumnSelectVector::DataReadType read_type;
     while (size_t run_length = select_vector.get_next_run(&read_type)) {
         switch (read_type) {
@@ -373,7 +374,8 @@ Status FixLengthDecoder::_decode_datetime64(MutableColumnPtr& doris_column,
                 v.from_unixtime(date_value / _decode_params->second_mask, *_decode_params->ctz);
                 if constexpr (std::is_same_v<CppType, DateV2Value<DateTimeV2ValueType>>) {
                     // nanoseconds will be ignored.
-                    v.set_microsecond((date_value % _decode_params->second_mask) * scale_to_micro);
+                    v.set_microsecond((date_value % _decode_params->second_mask) *
+                                      _decode_params->scale_to_nano_factor / 1000);
                     // TODO: the precision of datetime v1
                 }
                 _FIXED_SHIFT_DATA_OFFSET();
