@@ -656,7 +656,7 @@ Status HashJoinNode::_materialize_build_side(RuntimeState* state) {
     RETURN_IF_ERROR(child(1)->open(state));
 
     SCOPED_TIMER(_build_timer);
-    MutableBlock mutable_block(child(1)->row_desc().tuple_descriptors());
+    MutableBlock mutable_block;
 
     uint8_t index = 0;
     int64_t last_mem_used = 0;
@@ -669,8 +669,9 @@ Status HashJoinNode::_materialize_build_side(RuntimeState* state) {
         Block block;
         // If eos or have already met a null value using short-circuit strategy, we do not need to pull
         // data from data.
+        _build_col_ids.resize(_build_expr_ctxs.size());
         while (!eos && !_short_circuit_for_null_in_probe_side) {
-            block.clear_column_data();
+            release_block_memory(block, 1);
             RETURN_IF_CANCELLED(state);
 
             RETURN_IF_ERROR_AND_CHECK_SPAN(child(1)->get_next_after_projects(state, &block, &eos),
@@ -679,6 +680,8 @@ Status HashJoinNode::_materialize_build_side(RuntimeState* state) {
             _mem_used += block.allocated_bytes();
 
             if (block.rows() != 0) {
+                RETURN_IF_ERROR(_do_evaluate(block, _build_expr_ctxs, *_build_expr_call_timer,
+                                             _build_col_ids));
                 SCOPED_TIMER(_build_side_merge_block_timer);
                 RETURN_IF_CATCH_BAD_ALLOC(mutable_block.merge(block));
             }
@@ -889,8 +892,6 @@ Status HashJoinNode::_process_build_block(RuntimeState* state, Block& block, uin
     ColumnRawPtrs raw_ptrs(_build_expr_ctxs.size());
 
     ColumnUInt8::MutablePtr null_map_val;
-    std::vector<int> res_col_ids(_build_expr_ctxs.size());
-    RETURN_IF_ERROR(_do_evaluate(block, _build_expr_ctxs, *_build_expr_call_timer, res_col_ids));
     if (_join_op == TJoinOp::LEFT_OUTER_JOIN || _join_op == TJoinOp::FULL_OUTER_JOIN) {
         _convert_block_to_null(block);
     }
@@ -898,7 +899,7 @@ Status HashJoinNode::_process_build_block(RuntimeState* state, Block& block, uin
     //  so we have to initialize this flag by the first build block.
     if (!_has_set_need_null_map_for_build) {
         _has_set_need_null_map_for_build = true;
-        _set_build_ignore_flag(block, res_col_ids);
+        _set_build_ignore_flag(block, _build_col_ids);
     }
     if (_short_circuit_for_null_in_build_side || _build_side_ignore_null) {
         null_map_val = ColumnUInt8::create();
@@ -906,7 +907,7 @@ Status HashJoinNode::_process_build_block(RuntimeState* state, Block& block, uin
     }
 
     // Get the key column that needs to be built
-    Status st = _extract_join_column<true>(block, null_map_val, raw_ptrs, res_col_ids);
+    Status st = _extract_join_column<true>(block, null_map_val, raw_ptrs, _build_col_ids);
 
     st = std::visit(
             Overload {
