@@ -300,10 +300,6 @@ void VNodeChannel::try_send_block(RuntimeState* state) {
             _add_block_closure->clear_in_flight();
             return;
         }
-        {
-            vectorized::Block tmp_block(*request.mutable_block());
-            CHECK(block.rows() == tmp_block.rows());
-        }
         if (compressed_bytes >= double(config::brpc_max_body_size) * 0.95f) {
             LOG(WARNING) << "send block too large, this rpc may failed. send size: "
                          << compressed_bytes << ", threshold: " << config::brpc_max_body_size
@@ -430,7 +426,7 @@ VOlapTableSink::VOlapTableSink(ObjectPool* pool, const RowDescriptor& row_desc,
         : OlapTableSink(pool, row_desc, texprs, status) {
     _is_vectorized = true;
     // From the thrift expressions create the real exprs.
-    vectorized::VExpr::create_expr_trees(pool, texprs, &_output_vexpr_ctxs);
+    *status = vectorized::VExpr::create_expr_trees(pool, texprs, &_output_vexpr_ctxs);
     _name = "VOlapTableSink";
 }
 
