@@ -1406,20 +1406,6 @@ public class Config extends ConfigBase {
     @ConfField(mutable = true, masterOnly = true)
     public static boolean recover_with_empty_tablet = false;
 
-    /**
-     * In some scenarios, there is an unrecoverable metadata problem in the cluster,
-     * and the visibleVersion of the data does not match be. In this case, it is still
-     * necessary to restore the remaining data (which may cause problems with the correctness of the data).
-     * This configuration is the same as` recover_with_empty_tablet` should only be used in emergency situations
-     * This configuration has three values:
-     *   disable : If an exception occurs, an error will be reported normally.
-     *   ignore_version: ignore the visibleVersion information recorded in fe partition, use replica version
-     *   ignore_all: In addition to ignore_version, when encountering no queryable replica,
-     *   skip it directly instead of throwing an exception
-     */
-    @ConfField(mutable = true, masterOnly = true)
-    public static String recover_with_skip_missing_version = "disable";
-
     /**
      * Whether to add a delete sign column when create unique table
      */
@@ -1881,6 +1867,12 @@ public class Config extends ConfigBase {
     @ConfField(mutable = true, masterOnly = false)
     public static long hive_metastore_client_timeout_second = 10;
 
+    /**
+     * Whether to load default config files when creating hive metastore client.
+     */
+    @ConfField(mutable = true, masterOnly = false)
+    public static boolean load_default_conf_for_hms_client = true;
+
     /**
      * Used to determined how many statistics collection SQL could run simultaneously.
      */
@@ -1909,7 +1901,7 @@ public class Config extends ConfigBase {
     public static long max_hive_partition_cache_num = 100000;
 
     @ConfField(mutable = false, masterOnly = false)
-    public static long max_hive_table_catch_num = 1000;
+    public static long max_hive_table_cache_num = 1000;
 
     @ConfField(mutable = false, masterOnly = false)
     public static short max_hive_list_partition_num = -1;
@@ -1940,7 +1932,7 @@ public class Config extends ConfigBase {
      * For external schema cache and hive meta cache.
      */
     @ConfField(mutable = false, masterOnly = false)
-    public static long external_cache_expire_time_minutes_after_access = 24 * 60; // 1 day
+    public static long external_cache_expire_time_minutes_after_access = 10; // 10 mins
 
     /**
      * Set session variables randomly to check more issues in github workflow
@@ -2047,5 +2039,23 @@ public class Config extends ConfigBase {
 
     @ConfField(mutable = true)
     public static boolean use_mysql_bigint_for_largeint = false;
+
+    /** 
+    * the max package size fe thrift server can receive,avoid accepting error or too large package causing OOM,default 20M
+    */
+    @ConfField
+    public static int fe_thrift_max_pkg_bytes = 20000000;
+
+    @ConfField(mutable = true, masterOnly = true)
+    public static int backup_upload_task_num_per_be = 3;
+
+    @ConfField(mutable = true, masterOnly = true)
+    public static int restore_download_task_num_per_be = 3;
+
+    @ConfField(mutable = false, masterOnly = false)
+    public static int http_sql_submitter_max_worker_threads = 2;
+
+    @ConfField(mutable = false, masterOnly = false)
+    public static int http_load_submitter_max_worker_threads = 2;
 }
 
