@@ -80,7 +80,7 @@ public class CreateViewStmt extends BaseViewStmt {
             Analyzer viewAnalyzer = new Analyzer(analyzer);
             viewDefStmt.analyze(viewAnalyzer);
 
-            createColumnAndViewDefs(analyzer);
+            createColumnAndViewDefs(viewAnalyzer);
         } finally {
             // must reset this flag, otherwise, all following query statement in this connection
             // will not do constant fold for nondeterministic functions.
